VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "NCommands"
Attribute VB_Creatable = False
Attribute VB_Exposed = True
Option Explicit
#If Win32 Then
DefLng A-Z
#Else
DefInt A-Z
#End If

'$ Uses ADDINMAN.CLS CMD.CLS

''@B Members
Private cmds As New Collection
Private rman As CAddInManager
Private rap As Application
''@E Members

Private fLogFile As Boolean
Private sLogFile As String
Private nLogFile As Integer

''@B Create
Function Create(rmanA As CAddInManager) As Boolean
    On Error GoTo CreateFail
    Set rman = rmanA
    Set rap = rman.Application
    
    ' Initialize internal commands here
    If Not Add(sCmdCustomize, _
           "Display Customize dialog box") Then GoTo CreateFail
#If Win32 Then
    If Not Add(sCmdLogFile, _
           "Write IDE events to a log file") Then GoTo CreateFail
#End If
    If Not Add(sCmdShowInfo, _
           "Display current state of the IDE") Then GoTo CreateFail

    ' Initialize commands from settings
    If Not AddSettings Then GoTo CreateFail
    
#If Win32 Then
    ' Restore last LogFile setting
    fLogFile = GetSetting(App.ExeName, sCmdLogFile, sValue, False)
#End If
    
    Create = True
CreateFail:
End Function
''@E Create

Sub Destroy()
    On Error Resume Next
    ' Close LogFile if it is present
    If nLogFile Then nLogFile = 0: fLogFile = False
    ' Remove all the commands
    Dim v As Variant
    For Each v In cmds
        Remove v.Name, True
    Next
End Sub

''@B Add
Function Add(sName As String, _
             sDescription As String, _
             Optional vExeName As Variant, _
             Optional vArguments As Variant, _
             Optional vInitDir As Variant, _
             Optional vPromptArg As Variant, _
             Optional vDisableCode As Variant) As Boolean
             
    Dim rcmd As CCommand
    On Error GoTo AddFail
    Set rcmd = New CCommand
    With rcmd
        .Name = sName
        .Description = sDescription
        If Not IsMissing(vExeName) Then .ExeName = vExeName
        If Not IsMissing(vArguments) Then .Arguments = vArguments
        If Not IsMissing(vInitDir) Then .InitDir = vInitDir
        If Not IsMissing(vPromptArg) Then .PromptArg = vPromptArg
        If Not IsMissing(vExeName) Then .DisableCode = vDisableCode
        cmds.Add rcmd, sName
        If .ExeName <> sEmpty Then
            ' Save external commands to Registry or INI file
            Serialize rcmd
        End If
    End With
    Add = True
AddFail:
End Function
''@E Add

''@B Remove
Sub Remove(sName As String, Optional vKeep As Variant)
    cmds.Remove sName
    If IsMissing(vKeep) Then vKeep = False
    ' Unregister
    If Not vKeep Then
        DeleteSetting App.ExeName, sCmdSection, sName
    End If
End Sub
''@E Remove


    ''@B Execute1
Function Execute(rcmd As CCommand) As Variant
    If rcmd.ExeName = sEmpty Then
        ' Handle internal commands
        Select Case rcmd.Name
        Case sCmdCustomize
            Execute = Customize
        Case sCmdLogFile
            LogFile = Not LogFile
            Execute = True
        Case sCmdShowInfo
            Execute = ShowInfo
        Case Else
            BugAssert False
        End Select
    ''@B Execute2
    Else
        ' Handle external commands
    ''@E Execute1...
        ' Execute won't work for "new" project with no filenames
        If GetFileExt(CodeFile) = sEmpty Then Exit Function
        
        ' Resembles Save Project menu item, but doesn't save project file
        SaveCodeFiles
        Dim s As String, sName As String, sFile As String
        ' Handle arguments
        If rcmd.PromptArg Then
            s = InputBox("Enter command-line arguments: ")
        Else
            s = rcmd.Arguments
        End If
        s = ExpandArgs(s)
        
        ' Set properties
        If rcmd.DisableCode Then EnableWindow HWndCode, False
        Dim hProg ' As SysInt
        hProg = Shell(rcmd.ExeName & sSpace & s, vbNormalFocus)
        If rcmd.DisableCode Then
            Do While IsRunning(hProg)
                DoEvents
            Loop
            EnableWindow HWndCode, True
            Component.Reload
        End If
    End If
    ''@E Execute2
  
End Function
''@E Execute

Property Get Application() As Application
    Set Application = rap
End Property

Property Get Parent() As CAddInManager
    Set Parent = rman
End Property

Property Get Manager() As CAddInManager
    Set Manager = rman
End Property

Property Get Count() As Integer
    Count = cmds.Count
End Property

Function Item(v As Variant) As CCommand
    On Error Resume Next
    Set Item = cmds(v)
    ' Return Nothing for failure
End Function

Function Items() As Collection
    Set Items = cmds
End Function

Property Get Project() As ProjectTemplate
    Set Project = rap.ActiveProject
End Property

Function Component(Optional vIndex As Variant) As Component
    If IsMissing(vIndex) Then
        Set Component = rap.ActiveProject.SelectedComponents(0)
    Else
        Set Component = rap.ActiveProject.Components(vIndex)
    End If
End Function

Property Get Components() As Components
    Set Components = rap.ActiveProject.Components
End Property

Function CodeFile(Optional vIndex As Variant) As String
    If IsMissing(vIndex) Then
        CodeFile = rap.ActiveProject.SelectedComponents(0).FileNames(0)
    Else
        CodeFile = Components(vIndex).FileNames(0)
    End If
End Function

Property Get CodeName() As String
    CodeName = rap.ActiveProject.SelectedComponents(0).Name
End Property

Property Get ProjectName() As String
    Dim s As String
    s = Project.FileName
    If s = sEmpty Then s = "Project1" Else s = GetFileBase(s)
    ProjectName = s
End Property

''@B HWndCode
Function HWndCode(Optional vName As Variant) As Long
    If IsMissing(vName) Then vName = CodeName
    HWndCode = FindWindow("VBCodeWin", vName)
End Function
''@E HWndCode

Function HWndProject() As Long
    HWndProject = FindWindow("PROJECT", ProjectName)
End Function

Function HWndProperties() As Long
    HWndProperties = VBFindTopWindow("wndclass_pbrs", _
                                     "Properties - " & CodeName)
End Function

Function HWndVB() As Long
    HWndVB = VBFindTopWindow("wndclass_desked_gsk", ProjectName & "*")
End Function

'' Private procedures

Private Function AddSettings() As Boolean
    AddSettings = True  ' Assume success
    Dim sName As String, sValue As String, sDescription As String
    Dim sExeName As String, sArgs As String, sInitDir As String
    Dim fPromptArg As Boolean, fDisableCode As Boolean
    
    ' Retrieve all settings into 2D array of setting names and values
    Dim va As Variant, i As Integer, iCur As Integer
    va = GetAllSettings(App.ExeName, sCmdSection)
    ' Quit if no commands registered
    ' Note: You must use "IsEmpty(v)" rather than more intuitive "v = Empty"
    If IsEmpty(va) Then Exit Function
    ' Process each setting
    For i = LBound(va, 1) To UBound(va, 1)
        sName = va(i, 0)
        sValue = va(i, 1)
        iCur = 1
        sDescription = NextStr(sValue, iCur)
        sExeName = NextStr(sValue, iCur)
        sArgs = NextStr(sValue, iCur)
        sInitDir = NextStr(sValue, iCur)
        fPromptArg = NextStr(sValue, iCur)
        fDisableCode = NextStr(sValue, iCur)
        If Not Add(sName, sDescription, sExeName, sArgs, sInitDir, fPromptArg, fDisableCode) Then
            AddSettings = False
            Exit Function
        End If
    Next
End Function

Private Function NextStr(sStr As String, iPos As Integer) As String
    Dim i As Integer
    i = iPos
    iPos = InStr(i, sStr, sComma)
    If iPos Then
        NextStr = Mid$(sStr, i, iPos - i)
    Else
        NextStr = Mid$(sStr, i)
    End If
    iPos = iPos + 1
End Function

Property Get LogFileName() As String
    LogFileName = sLogFile
End Property

Function SaveProject() As Variant
    SaveProject = MenuCall("Save Project")
End Function

Function MenuCall(sItem As String) As Variant
    MenuCall = False
    Dim menu As New CMenuList
    Dim ritem As CMenuItem
    menu.Create HWndVB
    If Not menu.Find(sItem, ritem) Then Exit Function
    If Not ritem.Execute Then Exit Function
    SaveProject = True
End Function

Sub SaveCodeFiles()
    Dim v As Variant, i As Integer
    For Each v In Components
        If v.IsDirty Then v.SaveAs v.FileNames(0)
    Next
End Sub

Private Function ExpandArgs(sArgs As String) As String
    Dim sFile As String
    sFile = CodeFile
    Dim s As String, sToken As String
    Dim sLeft As String, sRight As String
    Dim v1 As Variant, v2 As Variant
    Dim iDir As Long, iBase As Long, iExt As Long
    Const sSep = "$"
    sToken = GetQToken(sArgs, sSep)
    Do While sToken <> sEmpty
        sRight = sEmpty
        sLeft = sToken
        v1 = InStr(sToken, sSpace)
        If v1 Then
            sRight = Mid$(sToken, v1)
            sLeft = Left$(sToken, v1 - 1)
        End If
        Select Case UCase$(sLeft)
        Case "FILEFULL"
            s = s & sFile & sRight
        Case "FILE"
            s = s & GetFileBaseExt(sFile) & sRight
        Case "FILEBASE"
            s = s & GetFileBase(sFile) & sRight
        Case "FILEDIR"
            s = s & GetFileDir(sFile) & sRight
        Case "FILEEXT"
            s = s & GetFileExt(sFile) & sRight
        Case "FILES"
            For Each v1 In Project.Components
                For Each v2 In v1.FileNames
                    s = s & v2 & sSpace
                Next
            Next
            s = s & sRight
        Case "NAME"
            s = s & CodeName & sRight
        Case "NAMES"
            For Each v1 In Project.Components
                s = s & v1.Name & sSpace
            Next
            s = s & sRight
        Case "PROJECT"
            s = s & GetFileBaseExt(Project.FileName) & sRight
        Case "PROJECTFULL"
            s = s & Project.FileName & sRight
        Case "PATHPROJECT"
            s = s & GetFileDir(Project.FileName) & sRight
        Case "PATHEXE"
            s = s & GetFileDir(Application.FileName) & sRight
        Case "PATHLAST"
            s = s & Application.LastUsedPath & sRight
        Case Else
            s = s & sToken
        End Select
        ' Get Next token
        sToken = GetQToken(sEmpty, sSep)
    Loop
    ExpandArgs = RTrim$(s)
    
End Function

Sub Serialize(rcmd As CCommand)
With rcmd
    Dim s As String
    s = .Description & sComma & .ExeName & sComma & _
        .Arguments & sComma & .InitDir & sComma & _
        .PromptArg & sComma & .DisableCode
    SaveSetting App.ExeName, sCmdSection, .Name, s
End With
End Sub

Public Sub LogEntry(sEvent As String, sName As String, ByVal sData As String)
    Dim s As String
    s = FmtStr(sEvent, 30, True) & FmtStr(sName, 15, True) & "= " & sData
    BugMessage s
    If fLogFile Then
        If nLogFile = 0 Then
            sLogFile = NormalizePath(App.Path) & App.ExeName & ".ADI"
            If ExistFile(sLogFile) Then Kill sLogFile
            nLogFile = FreeFile
        End If
            
        ' Warning: multiple instances can overwrite log file
        Open sLogFile For Append Shared As nLogFile
        ' Challenge: Rewrite to give each instance its own log file
        ' Write to log file
        Print #nLogFile, s
        Close nLogFile
        ' Opening and closing is the only way to commit in Basic
    End If
End Sub

'' Start internal commands

Function Customize() As Variant
    Dim custom As New FCustomize
    custom.Create Me
    custom.Show vbModal
    Customize = True
End Function

Property Get LogFile() As Boolean
    LogFile = fLogFile
End Property

Property Let LogFile(fLogFileA As Boolean)
    fLogFile = fLogFileA
#If Win32 Then ' Avoid 16-bit bug
    rman.MenuItems.Item(sCmdLogFile).Checked = fLogFileA
#End If
    SaveSetting App.ExeName, sCmdLogFile, sValue, fLogFile
End Property

''@B ShowInfo
Function ShowInfo() As Variant
    Dim info As New fInfo
    Static fLog As Boolean
    If info.Create(Me) = False Then Exit Function
    info.LogMode = fLog
    info.Show vbModal
    fLog = info.LogMode
    Unload info
    ShowInfo = True
End Function
''@E ShowInfo
'
